@@ -127,22 +127,25 @@ sub out
     my ($shipout) = ($state ? sub {$out .= $_[0];} : sub {$fh->print($_[0]);});
     my (@gids) = sort {$a <=> $b} keys %{$self->{'val'}};
 
-    $fmt = 1; $grp = 1;
+    $fmt = 1; $grp = 1; $eff = 0;
     for ($i = 1; $i <= $#gids; $i++)
     {
         if ($self->{'val'}{$gids[$i]} < $self->{'val'}{$gids[$i-1]} && $self->{'cover'})
         {
             $fmt = 2;
             last;
-        } elsif ($gids[$i] == $gids[$i-1] + 1)
+        } elsif ($gids[$i] == $gids[$i-1] + 1 && ($self->{'cover'} || $self->{'val'}{$gids[$i]} == $self->{'val'}{$gids[$i-1]}))
         { $eff++; }
         else
-        { $grp++; }
+        {
+            $grp++;
+            $eff += $gids[$i] - $gids[$i-1] if (!$self->{'cover'});
+        }
     }
-    if ($self->{'cover'})
-    { $fmt = 2 if ($eff / $grp > 4); }
-    else
-    { $fmt = 2 if ($grp > 1); }
+#    if ($self->{'cover'})
+    { $fmt = 2 if ($eff / $grp > 3); }
+#    else
+#    { $fmt = 2 if ($grp > 1); }
     
     if ($fmt == 1 && $self->{'cover'})
     {
@@ -156,7 +159,7 @@ sub out
         foreach $g (@gids)
         {
             if ($g > $last + 1)
-            { &$shipout(pack('n*', 0 x ($g - $last - 1))); }
+            { &$shipout(pack('n*', (0) x ($g - $last - 1))); }
             &$shipout(pack('n', $self->{'val'}{$g}));
             $last = $g;
         }
@@ -201,7 +204,7 @@ sub out
 }
 
 
-=head2 $c->add($glyphid)
+=head2 $c->add($glyphid[, $class])
 
 Adds a glyph id to the coverage table incrementing the count so that each subsequent addition
 has the next sequential number. Returns the index number of the glyphid added
@@ -210,14 +213,76 @@ has the next sequential number. Returns the index number of the glyphid added
 
 sub add
 {
-    my ($self, $gid) = @_;
+    my ($self, $gid, $class) = @_;
     
     return $self->{'val'}{$gid} if (defined $self->{'val'}{$gid});
-    $self->{'val'}{$gid} = $self->{'count'};
-    return $self->{'count'}++;
+    if ($self->{'cover'})
+    {
+        $self->{'val'}{$gid} = $self->{'count'};
+        return $self->{'count'}++;
+    }
+    else
+    {
+        $self->{'val'}{$gid} = $class || '0';
+        $self->{'max'} = $class if ($class > $self->{'max'});
+        return $class;
+    }
 }
 
 
+=head2 $c->signtaure
+
+Returns a vector of all the glyph ids covered by this coverage table or class
+
+=cut
+
+sub signature
+{
+    my ($self) = @_;
+    my ($vec, $range, $size);
+
+if (0)
+{
+    if ($self->{'cover'})
+    { $range = 1; $size = 1; }
+    else
+    {
+        $range = $self->{'max'};
+        $size = 1;
+        while ($range > 1)
+        {
+            $size = $size << 1;
+            $range = $range >> 1;
+        }
+        $range = $self->{'max'} + 1;
+    }
+    foreach (keys %{$self->{'val'}})
+    { vec($vec, $_, $size) = $self->{'val'}{$_} > $range ? $range : $self->{'val'}{$_}; }
+    length($vec) . ":" . $vec;
+}
+    $vec = join(";", map{"$_,$self->{'val'}{$_}"} keys %{$self->{'val'}});
+}
+
+=head2 @map=$c->sort
+
+Sorts the coverage table so that indexes are in ascending order of glyphid.
+Returns a map such that $map[$new_index]=$old_index.
+
+=cut
+
+sub sort
+{
+    my ($self) = @_;
+    my (@res, $i);
+
+    foreach (sort {$a <=> $b} keys %{$self->{'val'}})
+    {
+        push(@res, $self->{'val'}{$_});
+        $self->{'val'}{$_} = $i++;
+    }
+    @res;
+}
+
 =head2 $c->out_xml($context)
 
 Outputs this coverage/class in XML
